/***************************************************************************
                          offlineimservice.h -  description
                             -------------------
    begin                : Thu Sep 07 2006
    copyright            : (C) 2006 by Diederik van der Boor
    email                : "vdboor" --at-- "codingdomain.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef OFFLINEIMSERVICE_H
#define OFFLINEIMSERVICE_H

#include "extra/httpsoapconnection.h"

class ChatMessage;
class QDateTime;

/**
 * Wrapper for SOAP calls to the MSN application directory service.
 *
 * @author Diederik van der Boor
 * @ingroup NetworkExtra
 */
class OfflineImService : public HttpSoapConnection
{
  Q_OBJECT

  public:  // public methods
    // The constructor
                         OfflineImService( const QString &authT, const QString &authP, QObject *parent = 0 );
    // The destructor
    virtual             ~OfflineImService();

    // Delete messages from the storage space
    void                 deleteMessages( const QStringList &messageIds );
    // Request an offline message.
    void                 getMessage( const QString &messageId, bool markAsRead = false );
    // Request the Mail-Data field over SOAP.
    void                 getMetaData();

  private:
    // Extract the e-mail address from an RFC822 formatted string.
    QString              extractRFC822Address( const QString &address );
    // Process the getMessage response
    void                 processGetMessageResult( QDomElement &resultRoot );
    // Process server responses
    void                 slotRequestFinished( QDomElement &resultRoot );

  private:
    // The passport cookie 't' value.
    QString              authT_;
    // The passport cookie 't' value. 
    QString              authP_;
    // The passport header to send with SOAP
    QString              passportCookieHeader_;

  signals:
    // The getMetaData() call returned results
    void                 metaDataReceived( QDomElement metaData );
    // The getMessage() call returned results.
    void                 messageReceived( const QString &messageId, const QString &from, const QString &to,
                                          const QDateTime &date, const QString &body,
                                          const QString &runId, int sequenceNum );
};

#endif
